module GoSso
  class ApplicationController < ActionController::Base
    protect_from_forgery with: :exception

    before_action do
      GoSso::Current.host = request.base_url
    end

    def auth
      session[:go_sso_referrer] = params[:redirect_url] || request.referrer || request.base_url
      if GoSso.test_mode?
        redirect_to go_sso_callback_url
      else
        redirect_to GoSso.authorize_url
      end
    end

    def callback
      token = GoSso.test_mode? ? GoSso.generate_fake_token : GoSso.get_token(params[:code])
      set_sso_token(token.token, expires_at: token.expires_at)
      GoSso.after_login.call(self)
      redirect_to session.delete(:go_sso_referrer) || request.base_url
    rescue OAuth2::Error => error
      render json: {
        code: error.code,
        description: error.description
      }
    end

    def logout
      set_sso_token(nil)
      redirect_to params[:redirect_url] || request.referrer || request.base_url
    end

    private
    def set_sso_token(token, expires_at: nil)
      if token
        session[:go_sso_token] = token
        session[:go_sso_token_expires_at] = expires_at
      else
        session[:go_sso_token] = session[:go_sso_token_expires_at] = nil
      end
    end
  end
end
