module GoSso
  module Controllers
    module Helpers
      extend ActiveSupport::Concern

      included do
        helper_method :current_sso_user

        rescue_from GoSso::FailedToOpenConnection do
          render json: {
            message: 'Failed to open connection to SSO server. Try again later.'
          }, status: 500
        end
      end

      def current_sso_user
        @current_sso_user ||= begin
          return nil unless session[:go_sso_token]
          return nil if sso_token_expired?
          user_json = GoSso.get_user_json(session[:go_sso_token])
          GoSso::User.new(user_json)
        end
      rescue OAuth2::Error
        nil
      end

      def authenticate_sso_user_permission
        unless current_sso_user
          return redirect_to(go_sso_auth_url(redirect_url: request.url))
        end

        unless current_sso_user.can_access?(GoSso.main_app_module_name)
          return render json: { message: 'access denied' }, status: 401
        end
      end

      def sso_token_expired?
        session[:go_sso_token_expires_at] < Time.now.to_i
      end
    end
  end
end